"""
    Theremin control program
"""

# import necessary modules
import sys
import math
import numpy
import ctypes as ctp
import dwfconstants as dwfct
from scipy.signal import hilbert

# import own modules
import device
import supply
import scope
import wavegen


class state:
    # application parameters
    sampling_frequency = 1000000.0
    buffer_size = 8192
    amplitude = 2.0
    frame_repeat = 5
    fixed_frequency = 30500.0


# set parameters in modules
scope.state.sampling_frequency = state.sampling_frequency
wavegen.state.sampling_frequency = state.sampling_frequency
scope.state.buffer_size = state.buffer_size
wavegen.state.buffer_size = state.buffer_size
wavegen.state.frame_repeat = state.frame_repeat

# load dynamic the SDK
dwf = device.load()
supply.dwf = dwf
scope.dwf = dwf
wavegen.dwf = dwf

# open device, send handle to modules
hdwf = device.open()
supply.hdwf = hdwf
scope.hdwf = hdwf
wavegen.hdwf = hdwf

try:
    # setup instruments
    supply.initialize(5.0, -5.0)
    supply.enable(True)
    scope.initialize()
    wavegen.initialize()

    # generate fixed-frequency sine wave
    samples = numpy.arange(0, state.buffer_size - 1)
    angular_velocity = 2.0 * math.pi * state.fixed_frequency / state.sampling_frequency
    fixed_signal = samples * 1.0
    for index in range(len(samples)):
        fixed_signal[index] = math.sin(angular_velocity * samples[index])
    mixed_signal = samples * 1.0

    # loop
    while True:
        # mix signals
        variable_signal = scope.acquisition(1)
        variable_maximum = max(variable_signal)
        for index in range(len(fixed_signal)):
            mixed_signal[index] = fixed_signal[index] + \
                (variable_signal[index] / variable_maximum)

        # detect the envelope
        analytic_signal = hilbert(mixed_signal)
        amplitude_envelope = numpy.abs(analytic_signal)

        # remove offset and limit the resulting values between +1 and -1
        envelope_average = sum(amplitude_envelope) / len(amplitude_envelope)
        ac_envelope = amplitude_envelope - envelope_average
        envelope_maximum = max(ac_envelope)
        ac_envelope = ac_envelope / envelope_maximum

        # get potentiometer information and calculate volume
        voltages = scope.acquisition(2)
        voltage_average = sum(voltages) / len(voltages)
        volume = voltage_average * 1.0 / 5.0

        # output the result on the waveform generator
        wavegen.generate(ac_envelope, volume * state.amplitude)

except KeyboardInterrupt:
    # exit on ctrl+c
    pass

finally:
    # stop the supplies and close the device
    supply.enable(False)
    device.close()
